@@ -18,9 +18,13 @@ from data_pipeline_api.registry.common import (
     get_data,
     DataRegistryTarget,
     get_remote_filesystem_and_path,
+    unique_dicts,
+    get_reference,
+    upload_to_storage,
 )
-from data_pipeline_api.registry.upload import upload_from_config
+from data_pipeline_api.registry.upload import upload_from_config, upload_to_text_table
 from data_pipeline_api.file_api import FileAPI
+from data_pipeline_api.registry.utils import get_remote_options
 
 logger = logging.getLogger(__name__)
 
@@ -43,8 +47,14 @@ def _get_input_url(
     :param token: personal access token
     :return: url reference to the input parameter data product version component
     """
+    namespace_ref = get_reference(
+        {DataRegistryField.name: namespace}, DataRegistryTarget.namespace, data_registry_url, token
+    )
+    if namespace_ref is None:
+        raise ValueError(f"No namespace found for {namespace}")
+
     query_data = {
-        DataRegistryField.namespace: namespace,
+        DataRegistryField.namespace: namespace_ref,
         DataRegistryField.name: data_product_name,
         DataRegistryField.version: version,
     }
@@ -76,34 +86,6 @@ def _verify_hash(filename: Path, access_calculated_hash: str) -> None:
         )
 
 
-def upload_to_storage(
-    remote_uri: str,
-    storage_options: Dict[str, Any],
-    data_directory: Path,
-    filename: Path,
-    upload_path: Optional[Union[str, Path]] = None,
-) -> str:
-    """
-    Uploads a file to the remote uri
-     
-    :param remote_uri: URI to the root of the storage
-    :param storage_options: (key, value) pairs that are passed to the remote storage, e.g. credentials 
-    :param data_directory: root of the data directory read from the access log
-    :param filename: file to upload
-    :param upload_path: optional override to the upload path of the file
-    :return: path of the file on the remote storage
-    """
-    protocol = urllib.parse.urlsplit(remote_uri).scheme
-    upload_path = upload_path or filename.absolute().relative_to(data_directory.absolute()).as_posix()
-    fs, path = get_remote_filesystem_and_path(protocol, remote_uri, upload_path, **storage_options)
-    if protocol in {"file", "ssh", "sftp"}:
-        fs.makedirs(Path(path).parent.as_posix(), exist_ok=True)
-    logger.info(f"Uploading {filename.as_posix()} to {path} on {remote_uri}")
-    fs.put(filename.as_posix(), path)
-    # some remote filesystems expect the root uri in the path, others don't, but the registry path doesn't
-    return path.replace(remote_uri, "")
-
-
 def _add_storage_root(
     posts: List[YamlDict], remote_uri: str, accessibility: int, data_registry_url: str, token: str
 ) -> Union[YamlDict, str]:
@@ -140,7 +122,7 @@ def _add_data_product_output_posts(
     posts: List[YamlDict],
     path: str,
     data_product_name: str,
-    namespace: str,
+    namespace_str: str,
     model_version_str: str,
     run_id: str,
     component_name: str,
@@ -154,7 +136,7 @@ def _add_data_product_output_posts(
     :param posts: List of posts to the data registry, will be modified
     :param path: StorageLocation path
     :param data_product_name: Name of the output data product
-    :param namespace: namespace that the data product is a member of
+    :param namespace_str: namespace that the data product is a member of
     :param model_version_str: version number of the model
     :param run_id: id of the run
     :param component_name: name of the output component
@@ -171,6 +153,7 @@ def _add_data_product_output_posts(
         },
     )
     obj = _create_target_data_dict(DataRegistryTarget.object, {DataRegistryField.storage_location: storage_location})
+    namespace = _create_target_data_dict(DataRegistryTarget.namespace, {DataRegistryField.name: namespace_str})
     data_product = _create_target_data_dict(
         DataRegistryTarget.data_product,
         {
@@ -187,7 +170,7 @@ def _add_data_product_output_posts(
             DataRegistryField.object: obj,
         },
     )
-    posts.extend([storage_location, obj, data_product, object_component])
+    posts.extend([storage_location, obj, namespace, data_product, object_component])
     logger.debug(f"Creating ObjectComponent: {object_component}")
     return object_component
 
@@ -230,18 +213,6 @@ def _add_model_run(
     posts.append(code_run)
 
 
-def unique_posts(posts: List[YamlDict]) -> List[YamlDict]:
-    """
-    Removes duplicate posts from the list of posts while maintaining their ordering.
-
-    :param posts: List of posts to the data registry, will be modified
-    :return: Unique list of posts to the data registry
-    """
-    set_of_yamls = {yaml.safe_dump(d): None for d in posts}
-    logger.info(f"Removed {len(posts) - len(set_of_yamls)} duplicate POSTs.")
-    return [yaml.safe_load(t) for t in set_of_yamls.keys()]
-
-
 def _get_accessibility(config):
     if all(int(event.get("accessibility", 0)) == 0 for event in config["io"]):
         return 0
@@ -293,13 +264,21 @@ def _add_code_repo(
     """
     storage_root_uri = to_github_uri(model_uri, model_git_sha)
 
+    if storage_root_uri != model_uri:
+        # if it's changed, it's now a github uri, we know the split to the first @ is going to be the root to sha split
+        root, sha_and_path = storage_root_uri.split("@", maxsplit=1)
+        path = "@" + sha_and_path
+    else:
+        root = storage_root_uri
+        path = "/"
+
     code_repo_storage = _create_target_data_dict(
-        DataRegistryTarget.storage_root, {DataRegistryField.name: model_uri, DataRegistryField.root: storage_root_uri,},
+        DataRegistryTarget.storage_root, {DataRegistryField.name: root, DataRegistryField.root: root, },
     )
     code_repo_location = _create_target_data_dict(
         DataRegistryTarget.storage_location,
         {
-            DataRegistryField.path: "/",
+            DataRegistryField.path: path,
             DataRegistryField.hash: model_git_sha,
             DataRegistryField.storage_root: code_repo_storage,
         },
@@ -325,6 +304,29 @@ def _add_code_repo(
     return code_repo_obj
 
 
+def _upload_file_to_text_table(
+        posts: List[YamlDict],
+        filename: Union[str, Path],
+        data_registry_url: str,
+        token: str
+) -> YamlDict:
+    """
+    for a given filename, uploads its contents to the text_file table and returns a reference to the object that will be
+    posted
+
+    :param posts: List of posts to the data registry, will be modified
+    :param filename: path to the file to upload
+    :param data_registry_url: base url of the data registry
+    :param token: personal access token
+    :return: object reference to the uploaded file
+    """
+    filename = Path(filename)
+    location = upload_to_text_table(filename, data_registry_url, token)
+    obj = _create_target_data_dict(DataRegistryTarget.object, {DataRegistryField.storage_location: location})
+    posts.append(obj)
+    return obj
+
+
 def _upload_file_to_storage(
     posts: List[YamlDict],
     filename: Union[str, Path],
@@ -368,6 +370,7 @@ def upload_model_run(
     storage_options: Dict[str, str],
     data_registry_url: str,
     token: str,
+    text_file_table: bool = True,
 ) -> None:
     """
     Reads the provided configuration files and then calls PATCH or POST with the data to the data registry as
@@ -381,6 +384,7 @@ def upload_model_run(
     :param storage_options: (key, value) pairs that are passed to the remote storage, e.g. credentials
     :param data_registry_url: base url of the data registry
     :param token: personal access token
+    :param text_file_table: If true, model_config and submission_script are uploaded to the text_file table in the data_registry
     """
     remote_uri_override = remote_uri_override or remote_uri
     config_filename = Path(config_filename)
@@ -395,7 +399,7 @@ def upload_model_run(
     namespace = config.get("namespace")
     model_version_str = config_yaml["model_version"]
     model_name = config_yaml["model_name"]
-    open_timestamp = config_yaml["open_timestamp"]
+    open_timestamp = config["open_timestamp"]
     model_git_sha = config["metadata"]["git_sha"]
     model_uri = config["metadata"]["uri"]
 
@@ -439,16 +443,20 @@ def upload_model_run(
             )
             outputs.append(object_component)
 
-    model_config_obj = _upload_file_to_storage(posts, model_config_filename, remote_uri, storage_options, storage_root)
-    submission_script_obj = _upload_file_to_storage(
-        posts, submission_script_filename, remote_uri, storage_options, storage_root
-    )
+    if text_file_table:
+        model_config_obj = _upload_file_to_text_table(posts, model_config_filename, data_registry_url, token)
+        submission_script_obj = _upload_file_to_text_table(posts, submission_script_filename, data_registry_url, token)
+    else:
+        model_config_obj = _upload_file_to_storage(posts, model_config_filename, remote_uri, storage_options, storage_root)
+        submission_script_obj = _upload_file_to_storage(
+            posts, submission_script_filename, remote_uri, storage_options, storage_root,
+        )
 
     _add_model_run(
         posts, run_id, open_timestamp, inputs, outputs, model_config_obj, submission_script_obj, code_repo,
     )
 
-    posts = unique_posts(posts)
+    posts = unique_dicts(posts)
 
     upload_from_config({"post": posts}, data_registry_url, token)
 
@@ -493,12 +501,19 @@ def upload_model_run(
     help=f"data registry access token. Defaults to {DATA_REGISTRY_ACCESS_TOKEN} env if not passed."
     f" access tokens can be created from the data registry's get-token end point",
 )
+@click.option(
+    "--text-file-table/--no-text-file-table", default=True,
+    help="Whether to upload the model-config and submission-script to the text_file table of the data registry "
+         "(default), or to the remote-uri"
+)
 def upload_model_run_cli(
-    config, model_config, submission_script, remote_uri, remote_option, remote_uri_override, data_registry, token
+    config, model_config, submission_script, remote_uri, remote_option, remote_uri_override, data_registry, token, text_file_table,
 ):
     configure_cli_logging()
     data_registry = data_registry or DEFAULT_DATA_REGISTRY_URL
-    remote_options = dict(remote_option) if remote_option else {}
+    remote_options = get_remote_options()
+    arg_remote_options = dict(remote_option) if remote_option else {}
+    remote_options.update(arg_remote_options)
     remote_uri_override = remote_uri_override or remote_uri
     if not token:
         raise ValueError(
@@ -514,6 +529,7 @@ def upload_model_run_cli(
         storage_options=remote_options,
         data_registry_url=data_registry,
         token=token,
+        text_file_table=text_file_table,
     )
 
 
